﻿Imports ZXing
Imports ZXing.Common
Imports ZXing.QrCode
Imports System.Collections.Generic

Module Module_Barcodes

    ' QR-CODES
    ' ---------------------------------------
    ' QR_CODE     = QRcode classic
    ' AZTEC       = QRcode centered
    ' DATA_MATRIX = QRcode centered with black left lines
    ' PDF_417     = QRcode with aspect similar to barcode

    ' BARCODES
    ' ---------------------------------------
    ' CODABAR  = Barcode with 1 or more numeric only digits
    ' CODE_128 = Barcode with 1 to 80 digits 
    ' CODE_39  = Barcode with 1 to 80 digits (letters = 2 digits)
    ' CODE_93  = Barcode with 1 to 80 digits (letters = 2 digits)
    ' EAN_13   = Barcode with 12 numeric digits 
    ' EAN_8    = Barcode with 7 numeric digits 
    ' ITF      = Barcode with 2,4,6,8... up to 80 numeric only digits
    ' MSI      = Barcode with 1 or more numeric only digits
    ' PLESSEY  = Barcode with 1 or more numeric only digits
    ' UPC_A    = Barcode with 11 numeric digits
    ' UPC_E    = Barcode with 7 numeric digits

    ' NOT WORKING
    ' --------------------------------------
    ' All_1D
    ' IMB
    ' MAXICODE
    ' PHARMA_CODE
    ' RSS_14
    ' RSS_EXPANDED
    ' UPC_EAN_EXTENSION

    ' DECODED OK
    ' --------------------------------------
    ' QR_CODE  
    ' AZTEC       
    ' DATA_MATRIX 
    ' PDF_417     
    ' CODABAR
    ' CODE_128 
    ' CODE_39
    ' CODE_93


    ' =================================================================
    '   TEST NAMESPACES
    ' =================================================================
    Private Sub TestNamespaces()
        'ZXing.Common.BitArray()
        'ZXing.Common.BitMatrix()
        'ZXing.Common.BitSource()
        'ZXing.Common.CharacterSetECI()
        'ZXing.Common.DecoderResult()
        'ZXing.Common.DecodingOptions()
        'ZXing.Common.DefaultGridSampler()
        'ZXing.Common.Detector()
        'ZXing.Common.Detector.MathUtils()
        'ZXing.Common.Detector.MonochromeRectangleDetector()
        'ZXing.Common.Detector.WhiteRectangleDetector()
        'ZXing.Common.DetectorResult()
        'ZXing.Common.ECI()
        'ZXing.Common.EncodingOptions()
        'ZXing.Common.GlobalHistogramBinarizer()
        'ZXing.Common.GridSampler()
        'ZXing.Common.HybridBinarizer()
        'ZXing.Common.PerspectiveTransform()
        'ZXing.Common.ReedSolomon.GenericGF()
        'ZXing.Common.ReedSolomon.ReedSolomonDecoder()
        'ZXing.Common.ReedSolomon.ReedSolomonEncoder()
        'ZXing.Common.StringUtils.GB2312
        'ZXing.Common.StringUtils.guessEncoding()
        'ZXing.Common.StringUtils.ReferenceEquals()
        'ZXing.Common.StringUtils.SHIFT_JIS
    End Sub


    ' =================================================================
    '   ENCODER
    ' =================================================================
    Private BW As BarcodeWriter = New BarcodeWriter()

    Friend Function Encode(ByVal txt As String) As Image
        Dim options As QrCodeEncodingOptions = New QrCodeEncodingOptions

        ' -----------------------------------------------
        options.Width = 250 ' 120
        options.Height = 250 ' 120
        ' -----------------------------------------------
        options.DisableECI = True
        options.CharacterSet = "UTF-8"
        ' ----------------------------------------------- working with QR_CODE
        'options.QrVersion = 1 ' 1 to 40 (default = 1)
        options.Margin = 0
        'options.ErrorCorrection = QrCode.Internal.ErrorCorrectionLevel.H ' L Q M H
        ' ----------------------------------------------- 
        'options.Hint = ZXing.EncodeHintType.AZTEC_LAYERS  ' (add dictionary)
        'options.GS1Format = True
        'options.PureBarcode = True
        ' -----------------------------------------------
        BW.Options = options

        BW.Format = ZXing.BarcodeFormat.QR_CODE

        Try
            Return New Bitmap(BW.Write(txt))
        Catch
            Return Nothing
        End Try
    End Function



    ' =================================================================
    '   DECODER
    ' =================================================================
    Private BR As BarcodeReader = New BarcodeReader()

    Friend Function Decode(ByVal bmp As Bitmap) As String
        Try
            BR.AutoRotate = True
            BR.TryInverted = True
            BR.Options.TryHarder = True

            Dim r As Result = BR.Decode(bmp)
            If r IsNot Nothing Then
                Return r.ToString
            Else
                Return ""
            End If
        Catch
            Return ""
        End Try
    End Function

    Friend Sub Decoder_SetTypes(ByVal DecodeTypes As String)
        Dim f As List(Of BarcodeFormat) = New List(Of BarcodeFormat)
        '
        Select Case DecodeTypes.Trim
            Case "All QR codes and BAR codes"
                f.Add(BarcodeFormat.QR_CODE)
                f.Add(BarcodeFormat.AZTEC)
                f.Add(BarcodeFormat.DATA_MATRIX)
                f.Add(BarcodeFormat.PDF_417)
                '
                f.Add(BarcodeFormat.CODE_128)
                f.Add(BarcodeFormat.CODE_39)
                f.Add(BarcodeFormat.CODE_93)
                '
                f.Add(BarcodeFormat.CODABAR)
                f.Add(BarcodeFormat.EAN_13)
                f.Add(BarcodeFormat.EAN_8)
                f.Add(BarcodeFormat.ITF)
                f.Add(BarcodeFormat.MSI)
                f.Add(BarcodeFormat.PLESSEY)
                f.Add(BarcodeFormat.UPC_A)
                f.Add(BarcodeFormat.UPC_E)
                '
                'f.Add(BarcodeFormat.All_1D)
                'f.Add(BarcodeFormat.IMB)
                'f.Add(BarcodeFormat.MAXICODE)
                'f.Add(BarcodeFormat.PHARMA_CODE)
                'f.Add(BarcodeFormat.RSS_14)
                'f.Add(BarcodeFormat.RSS_EXPANDED)
                'f.Add(BarcodeFormat.UPC_EAN_EXTENSION)
                '
            Case "All QR codes"
                f.Add(BarcodeFormat.QR_CODE)
                f.Add(BarcodeFormat.AZTEC)
                f.Add(BarcodeFormat.DATA_MATRIX)
                f.Add(BarcodeFormat.PDF_417)
            Case "QR_CODE"
                f.Add(BarcodeFormat.QR_CODE)
            Case "AZTEC"
                f.Add(BarcodeFormat.AZTEC)
            Case "DATA_MATRIX"
                f.Add(BarcodeFormat.DATA_MATRIX)
            Case "PDF_417"
                f.Add(BarcodeFormat.PDF_417)
                '
            Case "All alphanumeric BAR codes"
                f.Add(BarcodeFormat.CODE_128)
                f.Add(BarcodeFormat.CODE_39)
                f.Add(BarcodeFormat.CODE_93)
            Case "CODE_128"
                f.Add(BarcodeFormat.CODE_128)
            Case "CODE_39"
                f.Add(BarcodeFormat.CODE_39)
            Case "CODE_93"
                f.Add(BarcodeFormat.CODE_93)
                '
            Case "All numeric BAR codes"
                f.Add(BarcodeFormat.CODABAR)
                f.Add(BarcodeFormat.EAN_13)
                f.Add(BarcodeFormat.EAN_8)
                f.Add(BarcodeFormat.ITF)
                f.Add(BarcodeFormat.MSI)
                f.Add(BarcodeFormat.PLESSEY)
                f.Add(BarcodeFormat.UPC_A)
                f.Add(BarcodeFormat.UPC_E)
            Case "CODABAR"
                f.Add(BarcodeFormat.CODABAR)
            Case "EAN_13"
                f.Add(BarcodeFormat.EAN_13)
            Case "EAN_8"
                f.Add(BarcodeFormat.EAN_8)
            Case "ITF"
                f.Add(BarcodeFormat.ITF)
            Case "MSI"
                f.Add(BarcodeFormat.MSI)
            Case "PLESSEY"
                f.Add(BarcodeFormat.PLESSEY)
            Case "UPC_A"
                f.Add(BarcodeFormat.UPC_A)
            Case "UPC_E"
                f.Add(BarcodeFormat.UPC_E)
        End Select
        '
        BR.Options.PossibleFormats = f
    End Sub

    Friend Function GetDecoderComments(ByVal DecodeTypes As String) As String
        Dim s As String = ""
        '
        Select Case DecodeTypes.Trim
            Case "All QR codes and BAR codes"
                s = "Decode all the code types"
                ' ----------------------------------------------------------
            Case "All QR codes"
                s = "Decode the QR codes only"
            Case "QR_CODE"
                s = "QRcode classic"
            Case "AZTEC"
                s = "QRcode centered"
            Case "DATA_MATRIX"
                s = "QRcode centered with black left lines"
            Case "PDF_417"
                s = "QRcode with aspect similar to barcode"
                ' ----------------------------------------------------------
            Case "All alphanumeric BAR codes"
                s = "Decode the alphanumeric BAR codes only"
            Case "CODE_128"
                s = "Barcode with 1 to 80 digits "
            Case "CODE_39"
                s = "Barcode with 1 to 80 digits (letters = 2 digits)"
            Case "CODE_93"
                s = "Barcode with 1 to 80 digits (letters = 2 digits)"
                ' ----------------------------------------------------------
            Case "All numeric BAR codes"
                s = "Decode the numeric BAR codes only"
            Case "CODABAR"
                s = "Barcode with 1 or more numeric only digits"
            Case "EAN_13"
                s = "Barcode with 12 numeric digits "
            Case "EAN_8"
                s = "Barcode with 7 numeric digits "
            Case "ITF"
                s = "Barcode with 2,4,6,8... up to 80 numeric only digits"
            Case "MSI"
                s = "Barcode with 1 or more numeric only digits"
            Case "PLESSEY"
                s = "Barcode with 1 or more numeric only digits"
            Case "UPC_A"
                s = "Barcode with 11 numeric digits"
            Case "UPC_E"
                s = "Barcode with 7 numeric digits"
        End Select
        '
        Return s
    End Function

End Module
